// jstre.C  Version 2.20e  Copyleft (c) 杉本典夫
// Time-stamp: <2023-10-22 09:47:25 tac>
//
// 日本語文字列の各種処理を行う関数群
//
// ライブラリ：libfune.so.2.2
// コンパイル法：g++ -O -fPIC -c jstre.C

#include <fune.H>               // 共通関数用ヘッダファイル

using namespace fun;            // fun名前空間使用(名前空間std含む)

namespace                       // 名前無し名前空間(局所関数定義用)
{

int euc_kanadaku(uchar& high, uchar& low)
// EUC全角カタカナを濁音にして返す(局所関数)
{
  if (high != 0xA5) return 0;   // 全角カタカナ以外
  int daku = 0;
  switch (low)
  {
  case 0xAB :                   // "カ"
  case 0xAD :                   // "キ"
  case 0xAF :                   // "ク"
  case 0xB1 :                   // "ケ"
  case 0xB3 :                   // "コ"
  case 0xB5 :                   // "サ"
  case 0xB7 :                   // "シ"
  case 0xB9 :                   // "ス"
  case 0xBB :                   // "セ"
  case 0xBD :                   // "ソ"
  case 0xBF :                   // "タ"
  case 0xC1 :                   // "チ"
  case 0xC4 :                   // "ツ"
  case 0xC6 :                   // "テ"
  case 0xC8 :                   // "ト"
  case 0xCF :                   // "ハ"
  case 0xD2 :                   // "ヒ"
  case 0xD5 :                   // "フ"
  case 0xD8 :                   // "ヘ"
  case 0xDB : low += 1;  daku = 1; // "ホ"
    break;
  case 0xA6 : low = 0xF4;  daku = 1; // "ウ"→"ヴ"
    break;
  }
  return daku;                  // 0:無変換,1:変換
}

int euc_kanahdaku(uchar& high, uchar& low)
// EUC全角カタカナを半濁音にして返す(局所関数)
{
  if (high != 0xA5) return 0;   // 全角カタカナ以外
  int hdaku = 0;
  switch (low)
  {
  case 0xCF :                   // "ハ"
  case 0xD2 :                   // "ヒ"
  case 0xD5 :                   // "フ"
  case 0xD8 :                   // "ヘ"
  case 0xDB : low += 2;  hdaku = 1; // "ホ"
    break;
  }
  return hdaku;                 // 0:無変換,1:変換
}

int euc_kanasei(uchar& high, uchar& low)
// EUC全角カタカナ濁音・半濁音を清音にして返す(局所関数)
{
  if (high != 0xA5) return 0;   // 全角カタカナ以外
  int sei = 0;
  switch (low)
  {
  case 0xAC :                   // "ガ"
  case 0xAE :                   // "ギ"
  case 0xB0 :                   // "グ"
  case 0xB2 :                   // "ゲ"
  case 0xB4 :                   // "ゴ"
  case 0xB6 :                   // "ザ"
  case 0xB8 :                   // "ジ"
  case 0xBA :                   // "ズ"
  case 0xBC :                   // "ゼ"
  case 0xBE :                   // "ゾ"
  case 0xC0 :                   // "ダ"
  case 0xC2 :                   // "ヂ"
  case 0xC5 :                   // "ヅ"
  case 0xC7 :                   // "デ"
  case 0xC9 :                   // "ド"
  case 0xD0 :                   // "バ"
  case 0xD3 :                   // "ビ"
  case 0xD6 :                   // "ブ"
  case 0xD9 :                   // "ベ"
  case 0xDC : low -= 1;  sei = 1; // "ボ"
    break;
  case 0xF4 : low = 0xA6;  sei = 1; // "ヴ"→"ウ"
    break;
  case 0xD1 :                   // "パ"
  case 0xD4 :                   // "ピ"
  case 0xD7 :                   // "プ"
  case 0xDA :                   // "ペ"
  case 0xDD : low -= 2;  sei = 2; // "ポ"
    break;
  }
  return sei;           // 0:無変換,1:濁音→清音変換,2:半濁音→清音変換
}

int euc_ankzen(string& str, uchar high, uchar low)
// EUC半角ANKを全角に変換する(局所関数)
{
  if (str.size() > 1)           // 前の文字がある場合
  {
    uchar ch = str[str.size() - 2], cl = str[str.size() - 1];
    if (ch == 0xA5)             // 前の文字が漢字カタカナ
    {
      if (high == 0x8E && low == 0xDE) // 'ﾞ'(濁音)
      {
        if (euc_kanadaku(ch, cl) == 1) // 濁音化成功
        {
          char& chref = str[str.size() - 2];
          chref = ch;
          char& clref = str[str.size() - 1];
          clref = cl;
          return 2;             // 変換文字数を返す
        }
      }
      else if (high == 0x8E && low == 0xDF) // 'ﾟ'(半濁音)
      {
        if (euc_kanahdaku(ch, cl) == 1) // 半濁音化成功
        {
          char& chref = str[str.size() - 2];
          chref = ch;
          char& clref = str[str.size() - 1];
          clref = cl;
          return 2;             // 変換文字数を返す
        }
      }
    }
  }
  euchantozen(high, low);
  str += high;
  str += low;
  return 1;                     // 変換文字数を返す
}

int euc_ankhan(string& str, uchar high, uchar low)
// EUC全角ANKを半角ANKに変換する(局所関数)
{
  int sei = euc_kanasei(high, low);
  euczentohan(high, low);
  if (high == 0x00)             // AN１文字に変換
  { str += low; }
  else if (high == 0x8E)        // 半角カタカナに変換
  {
    str += high;
    str += low;
    if (sei == 1)               // 濁音記号'ﾞ'追加
    {
      str += 0x8E;
      str += 0xDE;
    }
    else if (sei == 2)          // 半濁音記号'ﾟ'追加
    {
      str += 0x8E;
      str += 0xDF;
    }
  }
  else                          // 無変換
  {
    str += high;
    str += low;
  }
  return 1;                     // 変換文字数を返す
}

int euc_strhantozen(string& str, char mode)
// EUC文字列中の半角ANKを全角ANKに変換する
{
  if (str.empty()) return 0;
  bool ancv = false, kanacv = false;
  if (mode == 'a') { ancv = true;  kanacv = true; } // ANK変換
  else if (mode == 'n') ancv = true; // カタカナ以外のANK変換
  else if (mode == 'k') kanacv = true; // カタカナだけ変換
  uchar cpre = '\0';
  bool kanji = false, kana = false;
  int exkanji = 0;
  string tstr;
  for (strsize_t i = 0; i < str.size(); ++i)
  {
    uchar c = str[i];
    if (kanji)                  // 漢字2バイト目
    {
      tstr += cpre;
      tstr += c;
      kanji = false;  cpre = '\0';
    }
    else if (exkanji > 1)       // 拡張漢字3バイト目
    {
      tstr += 0x8F;
      tstr += cpre;
      tstr += c;
      exkanji = 0;  cpre = '\0';
    }
    else if (exkanji == 1)      // 拡張漢字2バイト目
    { ++exkanji;  cpre = c; }
    else if (kana)              // 半角カタカナ2バイト目
    {
      if (kanacv)               // 半角カタカナ変換
      { euc_ankzen(tstr, cpre, c); }
      else
      {
        tstr += cpre;
        tstr += c;
      }
      kana = false;  cpre = '\0';
    }
    else if (iseuckanji(c))     // 漢字1バイト目
    { kanji = true;  cpre = c; }
    else if (iseucexkanji(c))   // 拡張漢字1バイト目
    { exkanji = 1;  cpre = c; }
    else if (iseuchkana(c))     // 半角カタカナ1バイト目
    { kana = true;  cpre = c; }
    else if (iscntrl(c))        // コントロール文字
    { tstr += c; }
    else                        // その他の1バイト文字
    {
      if (ancv) euc_ankzen(tstr, 0x00, c); // AN変換
      else tstr += c;
    }
  }
  str = tstr;
  return int(str.size());       // 変換後の文字数を返す
}

int euc_strzentohan(string& str, char mode)
// EUC文字列中の全角ANKを半角ANKに変換する
{
  if (str.empty()) return 0;
  bool ancv = false, kanacv = false;
  if (mode == 'a') { ancv = true;  kanacv = true; } // ANK変換
  else if (mode == 'n') ancv = true; // カタカナ以外のANK変換
  else if (mode == 'k') kanacv = true; // カタカナだけ変換
  uchar cpre = '\0';
  bool kanji = false, kana = false;
  int exkanji = 0;
  string tstr;
  for (strsize_t i = 0; i < str.size(); ++i)
  {
    uchar c = str[i];
    if (kanji)                  // 漢字２バイト目
    {
      if (iseuczkana(cpre, c))  // 全角カタカナ
      {
        if (kanacv)
        { euc_ankhan(tstr, cpre, c); }
        else
        {
          tstr += cpre;
          tstr += c;
        }
      }
      else
      {
        if (ancv)
        { euc_ankhan(tstr, cpre, c); }
        else
        {
          tstr += cpre;
          tstr += c;
        }
      }
      kanji = false;  cpre = '\0';
    }
    else if (exkanji > 1)       // 拡張漢字３バイト目
    {
      tstr += 0x8F;
      tstr += cpre;
      tstr += c;
      exkanji = 0;  cpre = '\0';
    }
    else if (exkanji == 1)      // 拡張漢字２バイト目
    { ++exkanji;  cpre = c; }
    else if (kana)              // 半角カタカナ２バイト目
    {
      tstr += cpre;
      tstr += c;
      kana = false;  cpre = '\0';
    }
    else if (iseuckanji(c))     // 漢字１バイト目
    { kanji = true;  cpre = c; }
    else if (iseucexkanji(c))   // 拡張漢字１バイト目
    { exkanji = 1;  cpre = c; }
    else if (iseuchkana(c))     // 半角カタカナ１バイト目
    { kana = true;  cpre = c; }
    else                        // １バイト文字
    { tstr += c; }
  }
  str = tstr;
  return int(str.size());       // 変換後の文字数を返す
}

int esu_strtrm(string& str, codetype cdtype)
// EUC/Shift-JIS/UTF-8コード文字列の前後の空白を削除する(全角スペース含む)
{
  if (str.empty()) return 0;    // ヌル文字列
  strsize_t sidx = string::npos, eidx = string::npos, tidx = string::npos;
  int nc = 0;
  string buf;
  for (strsize_t i = 0; i < str.size(); ++i)
  {
    unsigned char c = str[i];
    if (nc > 1)                 // マルチバイトコードの途中
    { buf += c;  --nc; }
    else if (nc == 1)           // マルチバイトコードの最後
    {
      buf += c;
      if (!iszspace(buf.c_str(), cdtype)) // 全角スペース以外
      {
        if (sidx == string::npos) sidx = tidx; // 開始位置記憶
        eidx = i;               // 終了位置記憶
      }
      buf.erase();  nc = 0;
    }
    else if (ismcode(c, cdtype)) // マルチバイトコード1バイト目
    { tidx = i;  buf = c;  nc = ncode(c, cdtype) - 1; }
    else if (!isspace(c) && !iscntrl(c)) // ANK空白扱い文字以外
    {
      if (sidx == string::npos) sidx = i;
      eidx = i;
    }
  }
  if (sidx == string::npos)     // 空白文字のみ
  { str.erase(); }
  else
  {
    str.erase(eidx + 1);
    string tstr(str.substr(sidx));
    str = tstr;
  }
  return int(str.size());       // 変換後の文字列を返す
}

int esu_strclen(const string& str, uint chartype, codetype cdtype)
// EUC/Shift-JIS/UTF-8コード文字列の指定された種類の文字数を返す
{
  if (str.empty()) return 0;    // ヌル文字列
  int nc = 0;
  int n = 0, nascii = 0, nhkana = 0, nkanji = 0, netc = 0;
  string buf;
  for (strsize_t i = 0; i < str.size(); ++i)
  {
    unsigned char c = str[i];
    if (nc > 1)                 // マルチバイトコードの途中
    { buf += c;  --nc; }
    else if (nc == 1)           // マルチバイトコードの最後
    {
      buf += c;
      if (ishkana(buf.c_str(), cdtype)) ++nhkana; // 半角カタカナ
      else ++nkanji;            // 漢字
      buf.erase();  nc = 0;
    }
    else if (ismcode(c, cdtype)) // マルチバイトコード1バイト目
    { buf = c;  nc = ncode(c, cdtype) - 1; }
    else if (isascii(c))        // 7ビットASCII文字
    { ++nascii; }
    else
    { ++netc; }
  }
  if (chartype == CHARwide) return nascii + nhkana + nkanji * 2;
  if (chartype & CHARascii) n += nascii;
  if (chartype & CHARhkana) n += nhkana;
  if (chartype & CHARkanji) n += nkanji;
  if (chartype & CHARetc) n += netc;
  return n;                     // 文字数を返す
}

} // 名前無し名前空間終わり

esctype fun::jisescseq(const string& str)
// JISエスケープシーケンスの検査
{
  if (str == "\x1B$B" || str == "\x1B$@" || str == "\x1B$(D") return kin;
  if (str == "\x1B(B" || str == "\x1B(J") return kout;
  return escnon;                // エスケープシーケンスではない
}

int fun::cdconv(const char* inbuf, size_t insize, codetype incode, char* outbuf, size_t outsize, codetype outcode)
// inbuf中の日本語コードを変換してoutbufに格納して返す
{
  if (inbuf == nullptr || outbuf == nullptr) return 0; // ヌル文字列
  const char* incodename = nullptr;
  const char* outcodename = nullptr;
  switch (langcode())           // 現在の言語コードを取得する
  {
  case euc : incodename = "EUC-JP";
    break;
  case utf8 : incodename = "UTF-8";
    break;
  default : incodename = "SJIS";
  }
  outcodename = incodename;
  switch (incode)           // 日本語コードタイプをiconv用文字列にする
  {
  case euc : incodename = "EUC-JP";
    break;
  case jis : incodename = "ISO-2022-JP";
    break;
  case sjis : incodename = "SJIS";
    break;
  case utf8 : incodename = "UTF-8";
  }
  switch (outcode)
  {
  case euc : outcodename = "EUC-JP";
    break;
  case jis : outcodename = "ISO-2022-JP";
    break;
  case sjis : outcodename = "SJIS";
    break;
  case utf8 : outcodename = "UTF-8";
  }
  iconv_t cd = iconv_open(outcodename, incodename); // 変換用にオープン
  if (cd == iconv_t(-1))
  {
    disperr("cdconv", "iconv open eror");
    return -1;
  }
  char* inptr = (char*)inbuf;
  char* outptr = outbuf;
  int nconv = iconv(cd, &inptr, &insize, &outptr, &outsize);
  *outptr = '\0';
  iconv_close(cd);
  if (nconv < 0) disperr("cdconv", "iconv conversion error->", inptr);
  return nconv;                 // 非可逆変換回数または-1を返す
}

int fun::streuctojis(string& str)
// 文字列中のEUCコードをJISコードに変換する
{
  if (str.empty()) return 0;    // ヌル文字列
  size_t insize = str.size(), outsize = str.size() * 4;
  char* outbuf = new char[outsize + 1]; // JIS文字列格納用バッファ
  outbuf[0] = '\0';
  int rconv = cdconv(str.c_str(), insize, euc, outbuf, outsize, jis);
  if (rconv >= 0) str = outbuf; // 変換エラーなし
  delete [] outbuf;
  return rconv;                 // 非可逆変換回数または-1を返す
}

int fun::streuctosjis(string& str)
// 文字列中のEUCコードをShift-JISコードに変換する
{
  if (str.empty()) return 0;    // ヌル文字列
  size_t insize = str.size(), outsize = str.size();
  char* outbuf = new char[outsize + 1]; // Shift-JIS文字列格納用バッファ
  outbuf[0] = '\0';
  int rconv = cdconv(str.c_str(), insize, euc, outbuf, outsize, sjis);
  if (rconv >= 0) str = outbuf; // 変換エラーなし
  delete [] outbuf;
  return rconv;                 // 非可逆変換回数または-1を返す
}

int fun::streuctoutf8(string& str)
// 文字列中のEUCコードをUTF-8コードに変換する
{
  if (str.empty()) return 0;    // ヌル文字列
  size_t insize = str.size(), outsize = str.size() * 4;
  char* outbuf = new char[outsize + 1]; // UTF-8文字列格納用バッファ
  outbuf[0] = '\0';
  int rconv = cdconv(str.c_str(), insize, euc, outbuf, outsize, utf8);
  if (rconv >= 0) str = outbuf; // 変換エラーなし
  delete [] outbuf;
  return rconv;                 // 非可逆変換回数または-1を返す
}

int fun::strjistoeuc(string& str)
// 文字列中のJISコードをEUCコードに変換する
{
  if (str.empty()) return 0;    // ヌル文字列
  size_t insize = str.size(), outsize = str.size() * 4;
  char* outbuf = new char[outsize + 1]; // EUC文字列格納用バッファ
  outbuf[0] = '\0';
  int rconv = cdconv(str.c_str(), insize, jis, outbuf, outsize, euc);
  if (rconv >= 0) str = outbuf; // 変換エラーなし
  delete [] outbuf;
  return rconv;                 // 非可逆変換回数または-1を返す
}

int fun::strsjistoeuc(string& str)
// 文字列中のShift-JISコードをEUCコードに変換する
{
  if (str.empty()) return 0;    // ヌル文字列
  size_t insize = str.size(), outsize = str.size() * 3;
  char* outbuf = new char[outsize + 1]; // EUC文字列格納用バッファ
  outbuf[0] = '\0';
  int rconv = cdconv(str.c_str(), insize, sjis, outbuf, outsize, euc);
  if (rconv >= 0) str = outbuf; // 変換エラーなし
  delete [] outbuf;
  return rconv;                 // 非可逆変換回数または-1を返す
}

int fun::strutf8toeuc(string& str)
// 文字列中のUTF-8コードをEUCコードに変換する
{
  if (str.empty()) return 0;    // ヌル文字列
  size_t insize = str.size(), outsize = str.size();
  const char* inptr = str.c_str();
  if (str[0] == (char)0xEF && str[1] == (char)0xBB && str[2] == (char)0xBF)
  { inptr = str.c_str() + 3;  insize -= 3; } // BOM(バイト順)あり
  char* outbuf = new char[outsize + 1]; // EUC文字列格納用バッファ
  outbuf[0] = '\0';
  int rconv = cdconv(inptr, insize, utf8, outbuf, outsize, euc);
  if (rconv >= 0) str = outbuf; // 変換エラーなし
  delete [] outbuf;
  return rconv;                 // 非可逆変換回数または-1を返す
}

int fun::strutf8tojis(string& str)
// 文字列中のUTF-8コードをJISコードに変換する
{
  if (str.empty()) return 0;    // ヌル文字列
  size_t insize = str.size(), outsize = str.size() * 4;
  const char* inptr = str.c_str();
  if (str[0] == (char)0xEF && str[1] == (char)0xBB && str[2] == (char)0xBF)
  { inptr = str.c_str() + 3;  insize -= 3; } // BOM(バイト順)あり
  char* outbuf = new char[outsize + 1]; // JIS文字列格納用バッファ
  outbuf[0] = '\0';
  int rconv = cdconv(inptr, insize, utf8, outbuf, outsize, jis);
  if (rconv >= 0) str = outbuf; // 変換エラーなし
  delete [] outbuf;
  return rconv;                 // 非可逆変換回数または-1を返す
}

int fun::strutf8tosjis(string& str)
// 文字列中のUTF-8CコードをShift-JISコードに変換する
{
  if (str.empty()) return 0;    // ヌル文字列
  size_t insize = str.size(), outsize = str.size();
  const char* inptr = str.c_str();
  if (str[0] == (char)0xEF && str[1] == (char)0xBB && str[2] == (char)0xBF)
  { inptr = str.c_str() + 3;  insize -= 3; } // BOM(バイト順)あり
  char* outbuf = new char[outsize + 1]; // Shift-JIS文字列格納用バッファ
  outbuf[0] = '\0';
  int rconv = cdconv(inptr, insize, utf8, outbuf, outsize, sjis);
  if (rconv >= 0) str = outbuf; // 変換エラーなし
  delete [] outbuf;
  return rconv;                 // 非可逆変換回数または-1を返す
}

int fun::strjistoutf8(string& str)
// 文字列中のJISコードをUTF-8コードに変換する
{
  if (str.empty()) return 0;    // ヌル文字列
  size_t insize = str.size(), outsize = str.size() * 4;
  char* outbuf = new char[outsize + 1]; // UTF-8文字列格納用バッファ
  outbuf[0] = '\0';
  int rconv = cdconv(str.c_str(), insize, jis, outbuf, outsize, utf8);
  if (rconv >= 0) str = outbuf; // 変換エラーなし
  delete [] outbuf;
  return rconv;                 // 非可逆変換回数または-1を返す
}

int fun::strsjistoutf8(string& str)
// 文字列中のShift-JISコードをUTF-8コードに変換する
{
  if (str.empty()) return 0;    // ヌル文字列
  size_t insize = str.size(), outsize = str.size() * 4;
  char* outbuf = new char[outsize + 1]; // EUC文字列格納用バッファ
  outbuf[0] = '\0';
  int rconv = cdconv(str.c_str(), insize, sjis, outbuf, outsize, utf8);
  if (rconv >= 0) str = outbuf; // 変換エラーなし
  delete [] outbuf;
  return rconv;                 // 非可逆変換回数または-1を返す
}

int fun::strhantozen(string& str, char mode, codetype cdtype)
// 文字列中の半角ANKを全角ANKに変換する(汎用)
{
  if (str.empty()) return 0;
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  int rconv = 0;
  switch (cdtype)               // 文字処理用にコードをEUCに変換する
  {
  case jis : rconv = strjistoeuc(str);
    break;
  case sjis : rconv = strsjistoeuc(str);
    break;
  case utf8 : rconv = strutf8toeuc(str);
  }
  euc_strhantozen(str, mode);
  if (rconv < 0) return rconv;  // コード変換エラーの時は元のまま変換する
  switch (cdtype)               // EUCを元のコードに戻す
  {
  case jis : rconv = streuctojis(str);
    break;
  case sjis : rconv = streuctosjis(str);
    break;
  case utf8 : rconv = streuctoutf8(str);
  }
  if (rconv < 0) return rconv;  // 変換エラー
  return int(str.size());       // 変換後の文字数を返す
}

int fun::strzentohan(string& str, char mode, codetype cdtype)
// 文字列中の全角ANKを半角ANKに変換する(汎用)
{
  if (str.empty()) return 0;
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  int rconv = 0;
  switch (cdtype)               // 文字処理用にコードをEUCに変換する
  {
  case jis : rconv = strjistoeuc(str);
    break;
  case sjis : rconv = strsjistoeuc(str);
    break;
  case utf8 : rconv = strutf8toeuc(str);
  }
  euc_strzentohan(str, mode);
  if (rconv < 0) return rconv;  // コード変換エラーの時は元のまま変換する
  switch (cdtype)               // EUCを元のコードに戻す
  {
  case jis : rconv = streuctojis(str);
    break;
  case sjis : rconv = streuctosjis(str);
    break;
  case utf8 : rconv = streuctoutf8(str);
  }
  if (rconv < 0) return rconv;  // 変換エラー
  return int(str.size());       // 変換後の文字数を返す
}

int fun::strtrm(string& str, codetype cdtype)
// 文字列の前後の空白を削除する(全角スペース含む、汎用)
{
  if (str.empty()) return 0;    // ヌル文字列
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  int rconv = 0;
  if (cdtype == jis)            // コードをUTF-8に変換してから処理する
  {
    rconv = strjistoutf8(str);
    esu_strtrm(str, utf8);
  }
  else                          // EUC/Shift-JIS/UTF-8はそのまま処理する
  { esu_strtrm(str, cdtype); }
  if (rconv < 0) return rconv; // コード変換エラーの時は元のまま処理する
  if (cdtype == jis) rconv = strutf8tojis(str); // 元のコードに戻す
  if (rconv < 0) return rconv;  // 変換エラー
  return int(str.size());       // 変換後の文字数を返す
}

int fun::strclen(const string& str, uint chartype, codetype cdtype)
// 文字列中の指定された種類の文字数を返す(汎用)
{
  if (str.empty()) return 0;    // ヌル文字列
  if (cdtype == codenon) cdtype = langcode(); // コードが指定されていない時
  if (cdtype == jis)            // コードをUTF-8に変換してから処理する
  {
    string tstr(str);
    if (strjistoutf8(tstr) >= 0) return esu_strclen(tstr, chartype, utf8);
    cdtype = langcode();        // 変換エラー時は省略時コードと仮定する
  }
  return esu_strclen(str, chartype, cdtype); // 文字数を返す
}
