// optione.C  Version 2.20e  Copyleft (c) 杉本典夫
// Time-stamp: <2023-10-25 15:11:54 tac>
//
// オプションを処理する共通関数
// オプション指定の一般形式は次のとおり。
//     <種類>[ = <内容>][ # <コメント>]
// 各文字列の間の空白扱い文字(ANKスペース、漢字スペース、タブ)は無視する。
// 「#」以下もコメントとして無視する。
// 最初が「#」で始まる場合は全体をコメントとして無視する。
// 内容に空白扱い文字を指定する場合は前後を""で囲む。
//
// ライブラリ：libfune.so.2.2
// コンパイル法：g++ -O -fPIC -c optione.C

#include <fune.H>               // 共通関数用ヘッダファイル

using namespace fun;            // fun名前空間使用(名前空間std含む)

int fun::getopt(string opt[], const string& str, char devc, char comc)
// 文字列からオプションの種類と指定内容を取り出し文字列配列に格納する
{
  if (opt == 0) return -1;      // 引数不適エラー
  opt[0] = str;  opt[1].erase();
  if (str.empty()) return 0;    // ヌル文字列
  getsubstr(opt[0], str, comc); // コメントまでを取り出す
  strtrm(opt[0]);
  if (opt[0].empty()) return 0; // コメントのみ
  string tstr(opt[0]);
  strsize_t idx = getsubstr(opt[0], tstr, devc);
  if (idx >= tstr.size()) return 1; // オプションの種類のみ取得
  tstr.erase(0, idx + 1);       // オプションの種類を削除する
  opt[1] = tstr;
  strtrm(opt[0]);  strtrm(opt[1]);
  return 2;                     // オプションの種類と内容を取得
}

int fun::badopt(const string& pname, const string& opstr, const vecstr& usage, double low, double up)
// オプションの指定法が間違っている時の処理
{
  disperr(pname, "wrong method of specifying option->", opstr);
  if (low != DBL_MAX) cerr << "  specified range:" << low << " - " << up << "\n";
  cerr << "\n";
  for (string usagei : usage) cerr << usagei << "\n";
  return -1;
}

int fun::nonopt(const string& pname, const string& opstr, const vecstr& usage)
// 指定されたオプションがなかった場合の処理
{
  if (opstr.empty() == false && opstr != "?")
  {
    disperr(pname, "no such option->", opstr);
    cerr << "\n";
  }
  for (string usagei : usage) cerr << usagei << "\n";
  return (opstr == "?") ? 1 : -1; // -?の時は1を返す
}

int fun::help(const string& pname, const vecstr& usage, const string& man)
// オンラインマニュアルの表示
{
  string mancom("man");         // 省略時マニュアル表示用コマンド
  if (man.empty() == false) mancom = man; // マニュアル表示コマンドの指定
  mancom += " " + pname;
  if (system(mancom.c_str()) != 0) // コマンド起動失敗
  {
    for (string usagei : usage) cerr << usagei << "\n";
    return -1;
  }
  return 1;                     // ヘルプファイルを表示したら1を返す
}
